package com.rdlze.radializebase.interfaces;

import java.io.IOException;
import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;

import org.apache.lucene.queryparser.classic.ParseException;

/**
 * Class responsible for querying in musicbrainz lucene indexes
 * @author kledilson
 *
 */
public interface MBSearchServerInterface extends Remote, Serializable{
	
	/**
	 * Searches artist and song on musicbrainz lucene indexes
	 * @param search String to be searched
	 * @param index 0 for artist and 1 for song
	 * @param hitsPerPage number of results for the search
	 * @return array of xml with the responses
	 * @throws ParseException
	 * @throws IOException
	 */
	public String[] search(String search, int index, int hitsPerPage) throws ParseException,
	IOException, RemoteException;
	
	/**
	 * Searches for artist and song given and string that may contains artist and song
	 * @param bigTitle String containing artist and song ex: black sabbath paranoid live 1982
	 * @param hitsPerPage number of results for the search
	 * @return array of xml with the responses
	 * @throws RemoteException
	 * @throws ParseException
	 * @throws IOException
	 */
	public String[] searchBigTitle(String bigTitle, int hitsPerPage) throws RemoteException, ParseException, IOException;
	
	/**
	 * Method for registering remotes components
	 * @param remoteObject
	 * @param nameObject
	 * @throws RemoteException
	 */
//	public void registerRemoteObject(Remote remoteObject,
//			String nameObject) throws RemoteException;
	
}
